@@ -40,13 +40,16 @@ try:
 except ImportError:
     from httplib import BadStatusLine
 import socket
-import subprocess
 import re
 import DistUpgrade.DistUpgradeCache
 from gettext import gettext as _
+try:
+    from launchpadlib.launchpad import Launchpad
+except ImportError:
+    Launchpad = None
 
 SYNAPTIC_PINFILE = "/var/lib/synaptic/preferences"
-CHANGELOGS_POOL = "https://changelogs.ubuntu.com/changelogs/pool/"
+CHANGELOGS_POOL = "http://changelogs.ubuntu.com/changelogs/pool/"
 CHANGELOGS_URI = CHANGELOGS_POOL + "%s/%s/%s/%s_%s/%s"
 
 
@@ -64,8 +67,6 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
 
     def __init__(self, progress, rootdir=None):
         apt.Cache.__init__(self, progress, rootdir)
-        # save for later
-        self.rootdir = rootdir
         # raise if we have packages in reqreinst state
         # and let the caller deal with that (runs partial upgrade)
         assert len(self.req_reinstall_pkgs) == 0
@@ -77,27 +78,12 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         self._initDepCache()
         self.all_changes = {}
         self.all_news = {}
-        self.pro_versions = {}
         # on broken packages, try to fix via saveDistUpgrade()
         if self._depcache.broken_count > 0:
             self.saveDistUpgrade()
         assert (self._depcache.broken_count == 0 and
                 self._depcache.del_count == 0)
-        # generate versioned_kernel_pkgs_regexp for later use
-        apt_versioned_kernel_pkgs = apt_pkg.config.value_list(
-            "APT::VersionedKernelPackages")
-        if apt_versioned_kernel_pkgs:
-            self.versioned_kernel_pkgs_regexp = re.compile("(" + "|".join(
-                ["^" + p for p in apt_versioned_kernel_pkgs]) + ")")
-            running_kernel_version = subprocess.check_output(
-                ["uname", "-r"], universal_newlines=True).rstrip()
-            self.running_kernel_pkgs_regexp = re.compile("(" + "|".join(
-                [("^" + p + ".*" + running_kernel_version)
-                 if not p.startswith(".*") else (running_kernel_version + p)
-                 for p in apt_versioned_kernel_pkgs]) + ")")
-        else:
-            self.versioned_kernel_pkgs_regexp = None
-            self.running_kernel_pkgs_regexp = None
+        self.launchpad = None
 
     def _dpkgJournalDirty(self):
         """
@@ -194,9 +180,9 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
 
     def _strip_epoch(self, verstr):
         " strip of the epoch "
-        vers_no_epoch = verstr.split(":")
-        if len(vers_no_epoch) > 1:
-            verstr = "".join(vers_no_epoch[1:])
+        l = verstr.split(":")
+        if len(l) > 1:
+            verstr = "".join(l[1:])
         return verstr
 
     def _get_changelog_or_news(self, name, fname, strict_versioning=False,
@@ -218,9 +204,9 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         srcver = self._strip_epoch(srcver_epoch)
         #print("bin: %s" % binver)
 
-        split_section = section.split("/")
-        if len(split_section) > 1:
-            src_section = split_section[0]
+        l = section.split("/")
+        if len(l) > 1:
+            src_section = l[0]
 
         # lib is handled special
         prefix = srcpkg[0]
@@ -287,6 +273,53 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
             alllines = alllines + line
         return alllines
 
+    def _extract_ppa_changelog_uri(self, name):
+        """Return the changelog URI from the Launchpad API
+
+        Return None in case of an error.
+        """
+        if not Launchpad:
+            logging.warning("Launchpadlib not available, cannot retrieve PPA "
+                            "changelog")
+            return None
+
+        cdt = self[name].candidate
+        for uri in cdt.uris:
+            if urlsplit(uri).hostname != 'ppa.launchpad.net':
+                continue
+            match = re.search('http.*/(.*)/(.*)/ubuntu/.*', uri)
+            if match is not None:
+                user, ppa = match.group(1), match.group(2)
+                break
+        else:
+            logging.error("Unable to find a valid PPA candidate URL.")
+            return
+
+        # Login on launchpad if we are not already
+        if self.launchpad is None:
+            self.launchpad = Launchpad.login_anonymously('update-manager',
+                                                         'production',
+                                                         version='devel')
+
+
+        archive = self.launchpad.archives.getByReference(
+            reference='~%s/ubuntu/%s' % (user, ppa)
+        )
+        if archive is None:
+            logging.error("Unable to retrieve the archive from the Launchpad "
+                          "API.")
+            return
+
+        spphs = archive.getPublishedSources(source_name=cdt.source_name,
+                                            exact_match=True,
+                                            version=cdt.version)
+        if not spphs:
+            logging.error("No published sources were retrieved from the "
+                          "Launchpad API.")
+            return
+
+        return spphs[0].changelogUrl()
+
     def _guess_third_party_changelogs_uri_by_source(self, name):
         pkg = self[name]
         deb_uri = pkg.candidate.uri
@@ -321,7 +354,7 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         self.get_changelog(name)
         try:
             lock.release()
-        except Exception as e:
+        except:
             pass
 
     def get_news(self, name):
@@ -333,14 +366,25 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         if news:
             self.all_news[name] = news
 
-    def _fetch_changelog_for_third_party_package(self, name):
+    def _fetch_changelog_for_third_party_package(self, name, origins):
+        # Special case for PPAs
+        changelogs_uri_ppa = None
+        for origin in origins:
+            if origin.origin.startswith('LP-PPA-'):
+                try:
+                    changelogs_uri_ppa = self._extract_ppa_changelog_uri(name)
+                    break
+                except Exception:
+                    logging.exception("Unable to connect to the Launchpad API.")
         # Try non official changelog location
         changelogs_uri_binary = \
             self._guess_third_party_changelogs_uri_by_binary(name)
         changelogs_uri_source = \
             self._guess_third_party_changelogs_uri_by_source(name)
         error_message = ""
-        for changelogs_uri in [changelogs_uri_binary, changelogs_uri_source]:
+        for changelogs_uri in [changelogs_uri_ppa,
+                               changelogs_uri_binary,
+                               changelogs_uri_source]:
             if changelogs_uri:
                 try:
                     changelog = self._get_changelog_or_news(
@@ -359,24 +403,16 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
                         "Please check your Internet connection.")
         self.all_changes[name] += error_message
 
-    # If the machine is not attached to Ubuntu Pro, Update Manager advertises
-    # the upgrades that would be available if it were attached.
-    # As that is unbeknownst to Apt, we need this map to show the correct
-    # version of each upgradable-if-pro-subscribed package.
-    def create_pro_cache(self, pro_pkgs):
-        for (name, version, _a, _a) in pro_pkgs:
-            self.pro_versions[name] = version
-
     def get_changelog(self, name):
         " get the changelog file from the changelog location "
         origins = self[name].candidate.origins
-        version = self.pro_versions.get(name, self[name].candidate.version)
         self.all_changes[name] = _("Changes for %s versions:\n"
                                    "Installed version: %s\n"
                                    "Available version: %s\n\n") % \
-            (name, getattr(self[name].installed, "version", None), version)
+            (name, getattr(self[name].installed, "version", None),
+             self[name].candidate.version)
         if self.CHANGELOG_ORIGIN not in [o.origin for o in origins]:
-            self._fetch_changelog_for_third_party_package(name)
+            self._fetch_changelog_for_third_party_package(name, origins)
             return
         # fixup epoch handling version
         srcpkg = self[name].candidate.source_name
