@@ -22,6 +22,8 @@
 
 from __future__ import absolute_import, print_function
 
+import gi
+gi.require_version("Gtk", "3.0")
 from gi.repository import Gtk
 from gi.repository import Gdk
 
@@ -30,9 +32,12 @@ warnings.filterwarnings(
     "ignore", "Accessed deprecated property", DeprecationWarning)
 
 import logging
+import datetime
 import dbus
 import os
 
+import HweSupportStatus.consts
+
 from gettext import gettext as _
 
 
@@ -137,61 +142,6 @@ class InternalDialog(BuilderDialog):
             self.main_container.add(content_widget)
         self.main_container.set_visible(bool(content_widget))
 
-    def _is_livepatch_supported(self):
-        di = distro_info.UbuntuDistroInfo()
-        codename = get_dist()
-        return di.is_lts(codename)
-
-    def on_livepatch_status_ready(self, active, cs, ps, fixes):
-        self.set_desc(None)
-
-        if not active:
-            if self._is_livepatch_supported() and \
-               self.settings_button and \
-               self.settings.get_int('launch-count') >= 4:
-                self.set_desc(_("<b>Tip:</b> You can use Livepatch with "
-                                "Ubuntu Pro to keep your computer more "
-                                "secure between restarts."))
-                self.settings_button.set_label(_("Settings & Pro…"))
-            return
-
-        needs_reschedule = False
-
-        if cs == "needs-check":
-            needs_reschedule = True
-        elif cs == "check-failed":
-            pass
-        elif cs == "checked":
-            if ps == "unapplied" or ps == "applying":
-                needs_reschedule = True
-            elif ps == "applied":
-                fixes = [fix for fix in fixes if fix.patched]
-                d = ngettext("%d Livepatch update applied since the last "
-                             "restart.",
-                             "%d Livepatch updates applied since the last "
-                             "restart.",
-                             len(fixes)) % len(fixes)
-                self.set_desc(d)
-            elif ps == "applied-with-bug" or ps == "apply-failed":
-                fixes = [fix for fix in fixes if fix.patched]
-                d = ngettext("%d Livepatch update failed to apply since the "
-                             "last restart.",
-                             "%d Livepatch updates failed to apply since the "
-                             "last restart.",
-                             len(fixes)) % len(fixes)
-                self.set_desc(d)
-            elif ps == "nothing-to-apply":
-                pass
-            elif ps == "unknown":
-                pass
-
-        if needs_reschedule:
-            self.lp_socket.get_status(self.on_livepatch_status_ready)
-
-    def check_livepatch_status(self):
-        self.lp_socket = LivePatchSocket()
-        self.lp_socket.get_status(self.on_livepatch_status_ready)
-
 
 class StoppedUpdatesDialog(InternalDialog):
     def __init__(self, window_main):
@@ -248,6 +198,23 @@ class DistUpgradeDialog(InternalDialog):
                  "--frontend=DistUpgradeViewGtk3%s" % extra_args)
 
 
+class HWEUpgradeDialog(InternalDialog):
+    def __init__(self, window_main):
+        InternalDialog.__init__(self, window_main)
+        self.set_header(_("New important security and hardware support "
+                          "update."))
+        if datetime.date.today() < HweSupportStatus.consts.HWE_EOL_DATE:
+            self.set_desc(_(HweSupportStatus.consts.Messages.HWE_SUPPORT_ENDS))
+        else:
+            self.set_desc(_(HweSupportStatus.consts.Messages.HWE_SUPPORT_HAS_ENDED))
+        self.add_settings_button()
+        self.add_button(_("_Install…"), self.install)
+        self.focus_button = self.add_button(Gtk.STOCK_OK,
+                                            self.window_main.close)
+
+    def install(self):
+        self.window_main.start_install(hwe_upgrade=True)
+
 class UnsupportedDialog(DistUpgradeDialog):
     def __init__(self, window_main, meta_release):
         DistUpgradeDialog.__init__(self, window_main, meta_release)
@@ -357,7 +324,7 @@ class NeedRestartDialog(InternalDialog):
             iface.RequestReboot()
         except dbus.DBusException:
             self._request_reboot_via_consolekit()
-        except Exception as e:
+        except:
             pass
 
     def _request_reboot_via_consolekit(self):
@@ -370,7 +337,7 @@ class NeedRestartDialog(InternalDialog):
             iface.Restart()
         except dbus.DBusException:
             self._request_reboot_via_logind()
-        except Exception as e:
+        except:
             pass
 
     def _request_reboot_via_logind(self):
