@@ -7,6 +7,8 @@ import os
 import tempfile
 from gettext import gettext as _
 
+import gi
+gi.require_version("Gtk", "3.0")
 from gi.repository import GObject
 # Extra GdkX11 import for pygobject bug #673396
 # https://bugzilla.gnome.org/show_bug.cgi?id=673396
